// Copyright (c) 97 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// mSQL v1.* driver fro EasySQL API
//
// mSQL v1.* is a lightweight SQL engine. Not very powerfull, but may be successfully
// used on something like 386dx40/8
// mSQL is available from:
//  ftp://bond.edu.au/pub/Minerva/msql
//  ftp://ftp.hughes.com/pub/msql/
//
// There is also version 2.0 of mSQL which supports more complete subset
// of SQL and some nice features. This driver is INCOMPATIBLE with 2.0
// version API. See `msql2' driver instead!
//
#include <stdlib.h>	// for setenv()
#include <unistd.h>	// for write()
#include <msql.h>
#include "DrvHeader.h"

///////////////////////////////////////////////////////////////////////////////
// Data structure
//
struct DS(data) : public easysql_error_reporting
{ int sock;
//
  DS(data)();
  ~DS(data)();
};

///////////////////////////////////////////////////////////////////////////////
// Creating mSQL driver instance
//
DS(data)::DS(data) ()
         :easysql_error_reporting(DriverName "_driver",D_IDRV)
         ,sock(-1)
{
}

///////////////////////////////////////////////////////////////////////////////
// Destroing mSQL driver instance
//
DS(data)::~DS(data) ()
{
}

///////////////////////////////////////////////////////////////////////////////
// Connecting to mSQL server
//
bool
DS(connect) (DS(data) *data,
             char const *(*values)(void const *, char const *),
             void const *that)
{
// If port/socket are set, tune mSQL API to they
//
  if(values(that,"port"))
   setenv("MSQL_TCP_PORT",values(that,"port"),1);
  else
   unsetenv("MSQL_TCP_PORT");
  if(values(that,"socket"))
   setenv("MSQL_UNIX_PORT",values(that,"socket"),1);
  else
   unsetenv("MSQL_UNIX_PORT");

// Checking db name presence
//
  char const *db=values(that,"database");
  if(!db)
   { data->set_error("connect(): database name required");
     return false;
   }

// Connecting to mSQL engine
//
  char const *host=values(that,"host");
  data->sock=msqlConnect((char *)host);
  if(data->sock==-1)
   { data->set_error("connect(host=\"%s\"): %s",host,msqlErrMsg);
     return false;
   }

// Selecting database
//
  if(msqlSelectDB(data->sock,(char *)db)==-1)
   { data->set_error("connect(db=\"%s\"): %s",db,msqlErrMsg);
     return false;
   }

  log(D_IDRV|D_L2,"Connected to mSQL engine (%s,%s), socket=%d",host,db,data->sock);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Checking connection
//
bool
DS(check_connection) (DS(data) *data)
{ if(data->sock==-1) return false;
  if(write(data->sock,data,0)!=0)
   { data->set_error("msql server has gone, closing (socket=%d)",data->sock);
     DS(drop)(data);
     return false;
   }
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Dropping connection to engine
//
bool
DS(drop) (DS(data) *data)
{ if(data->sock!=-1)
   { msqlClose(data->sock);
     data->sock=-1;
   }
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Sending query
//
bool
DS(query) (DS(data) *data, esqlResult &r, char const *q)
{ log(D_IDRV|D_L1,"query(\"%s\")",q);
  r=NULL;

  if(msqlQuery(data->sock,(char *)q)==-1)
   { data->set_error("query(\"%s\"): %s",q,msqlErrMsg);
     return false;
   }

  r=(esqlResult)msqlStoreResult(data->sock);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Freeing query results
//
bool
DS(rfree) (DS(data) *, esqlResult r)
{ msqlFreeResult((m_result *)r);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Fetching next record from query results
//
bool
DS(row) (DS(data) *, esqlRow &row, esqlResult r)
{ row=(esqlRow)msqlFetchRow((m_result *)r);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Driver's and engine version
//
char const *
DS(version) (DS(data) *data)
{ static char *ver=NULL;
  if(!ver) ver=(typeof(ver))mAlloc(100);
  snprintf(ver,100,"%s %s",DriverName,EASYSQL_VERSION);
  if(DS(check_connection)(data))
   { char const *v=msqlGetServerInfo(data->sock);
     unsigned p=msqlGetProtoInfo(data->sock);
     if(v && strlen(v)+strlen(ver)+10<100)
      snprintf(ver+strlen(ver),100-strlen(ver)-1," %u %s",p,v);
   }
  return ver;
}

///////////////////////////////////////////////////////////////////////////////
// Connection and engine status. Due to inability to get any
// human-readable status from the engine we will two predefined messages
//
char const *
DS(status) (DS(data) *data)
{ return DS(check_connection)(data) ? "Connection established" : "Not connected to engine";
}

///////////////////////////////////////////////////////////////////////////////
// Standart functions
//
#include "../DrvStd.h"

///////////////////////////////////////////////////////////////////////////////
R("$Id: msql.C,v 1.4 1997/10/23 18:40:24 am Exp $");
